"""Intent Definitions.

Defines intent categories (task type, purpose, tone, reasoning depth)
to inform model and parameter routing.

Connected modules:
  - routing.model_router
  - config.params_policy
  - service.GenAIService
"""